#include "posTestWin.h"
#include "ui_posTestWin.h"
#include "utils.h"
#include "globalParams.h"
#include <QPushButton>
#include <qDebug>

PosTestWin::PosTestWin(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::PosTestWin)
{
    ui->setupUi(this);

    ui->gpbRdTest->setVisible(false);
    ui->gpbRnTest->setVisible(true);
    ui->gpbRnSignal->setVisible(false);

    connect(this,SIGNAL(recvGsvSignal(QString)),ui->rnSignalPainterWdg,SLOT(recvGsvHandle(QString)),Qt::QueuedConnection);

    rnModel = new QStandardItemModel(this);
    rnSelect = new QItemSelectionModel(rnModel);
    rnModel->setColumnCount(10);
    ui->tableViewRn->setModel(rnModel);
    ui->tableViewRn->setSelectionModel(rnSelect);
    ui->tableViewRn->horizontalHeader()->setVisible(false);
    ui->tableViewRn->verticalHeader()->setVisible(false);
    //ui->tableViewRn->horizontalHeader()->setStretchLastSection(true);
    ui->tableViewRn->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    ui->tableViewRn->verticalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    ui->tableViewRn->setWordWrap(true);


    rdModel = new QStandardItemModel(this);
    rdSelect = new QItemSelectionModel(rdModel);
    rdModel->setColumnCount(6);
    ui->tableViewRd->setModel(rdModel);
    ui->tableViewRd->setSelectionModel(rdSelect);
    ui->tableViewRd->horizontalHeader()->setVisible(false);
    ui->tableViewRd->verticalHeader()->setVisible(false);
    //ui->tableViewRn->horizontalHeader()->setStretchLastSection(true);
    ui->tableViewRd->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    ui->tableViewRd->verticalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    ui->tableViewRd->setWordWrap(true);

    QFont font = ui->tableViewRn->font();
    font.setBold(true);
    QFont smallFont = font;
    //smallFont.setPointSize(font.pointSize()-2);
    smallFont.setBold(false);

    QList<QStandardItem *> aItemList;
    for(int r=0;r<9;r++){
        aItemList.clear();
        for(int c=0;c<11;c++){
            QStandardItem *aItem = new QStandardItem("");
            aItem->setTextAlignment(Qt::AlignCenter);
            aItem->setFont(smallFont);
            aItemList.append(aItem);
        }
        rnModel->insertRow(rnModel->rowCount(),aItemList);
        ui->tableViewRn->setSpan(r,0,1,2);
    }

    for(int r=0;r<6;r++){
        aItemList.clear();
        for(int c=0;c<6;c++){
            QStandardItem *aItem = new QStandardItem("");
            aItem->setTextAlignment(Qt::AlignCenter);
            aItem->setFont(smallFont);
            aItemList.append(aItem);
        }
        rdModel->insertRow(rdModel->rowCount(),aItemList);
        ui->tableViewRd->setSpan(r,0,1,2);
    }

    rnModel->item(0,0)->setText("定位模式");
    rnModel->item(0,0)->setFont(font);
    ui->tableViewRn->setSpan(0,0,2,2);

    smallFont.setPointSize(font.pointSize()-1);
    smallFont.setBold(true);
    QPushButton *b1cBtn = new QPushButton("B1频点C码定位");
    b1cBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    b1cBtn->setFont(smallFont);
    b1cBtn->setDefault(true);
    b1cBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(2,0),b1cBtn);
    connect(b1cBtn,SIGNAL(clicked()),this,SLOT(b1cBtn_clicked()),Qt::QueuedConnection);

    QPushButton *b3cBtn = new QPushButton("B3频点C码定位");
    b3cBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    b3cBtn->setFont(smallFont);
    b3cBtn->setDefault(true);
    b3cBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(3,0),b3cBtn);
    connect(b3cBtn,SIGNAL(clicked()),this,SLOT(b3cBtn_clicked()),Qt::QueuedConnection);

    QPushButton *b1b3cBtn = new QPushButton("B1+B3频点C码定位");
    b1b3cBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    b1b3cBtn->setFont(smallFont);
    b1b3cBtn->setDefault(true);
    b1b3cBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(4,0),b1b3cBtn);
    connect(b1b3cBtn,SIGNAL(clicked()),this,SLOT(b1b3cBtn_clicked()),Qt::QueuedConnection);

    QPushButton *gpsBtn = new QPushButton("GPS频点C码定位");
    gpsBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    gpsBtn->setFont(smallFont);
    gpsBtn->setDefault(true);
    gpsBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(5,0),gpsBtn);
    connect(gpsBtn,SIGNAL(clicked()),this,SLOT(gpsBtn_clicked()),Qt::QueuedConnection);

    QPushButton *b1pBtn = new QPushButton("B1频点P码定位");
    b1pBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    b1pBtn->setFont(smallFont);
    b1pBtn->setDefault(true);
    b1pBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(6,0),b1pBtn);
    connect(b1pBtn,SIGNAL(clicked()),this,SLOT(b1pBtn_clicked()),Qt::QueuedConnection);

    QPushButton *b3pBtn = new QPushButton("B3频点P码定位");
    b3pBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    b3pBtn->setFont(smallFont);
    b3pBtn->setDefault(true);
    b3pBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(7,0),b3pBtn);
    connect(b3pBtn,SIGNAL(clicked()),this,SLOT(b3pBtn_clicked()),Qt::QueuedConnection);

    QPushButton *compatibleBtn = new QPushButton("兼容定位");
    compatibleBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    compatibleBtn->setFont(smallFont);
    compatibleBtn->setDefault(true);
    compatibleBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(8,0),compatibleBtn);
    connect(compatibleBtn,SIGNAL(clicked()),this,SLOT(compatibleBtn_clicked()),Qt::QueuedConnection);

    rnModel->item(0,2)->setText("定位结果");
    rnModel->item(0,2)->setFont(font);
    ui->tableViewRn->setSpan(0,2,1,7);

    rnModel->item(1,2)->setText("时间");
    rnModel->item(1,2)->setFont(font);

    rnModel->item(1,3)->setText("经度");
    rnModel->item(1,3)->setFont(font);

    rnModel->item(1,4)->setText("纬度");
    rnModel->item(1,4)->setFont(font);

    rnModel->item(1,5)->setText("高程");
    rnModel->item(1,5)->setFont(font);

    rnModel->item(1,6)->setText("PDOP");
    rnModel->item(1,6)->setFont(font);

    rnModel->item(1,7)->setText("VDOP");
    rnModel->item(1,7)->setFont(font);

    rnModel->item(1,8)->setText("HDOP");
    rnModel->item(1,8)->setFont(font);

    rnModel->item(0,9)->setText("卫星状态");
    rnModel->item(0,9)->setFont(font);
    ui->tableViewRn->setSpan(0,9,1,2);

    rnModel->item(1,9)->setText("有效卫星");
    rnModel->item(1,9)->setFont(font);

    rnModel->item(1,10)->setText("信号");
    rnModel->item(1,10)->setFont(font);

    QPushButton *viewB1CBtn = new QPushButton("查看");
    viewB1CBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    viewB1CBtn->setFont(smallFont);
    viewB1CBtn->setDefault(true);
    viewB1CBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(2,10),viewB1CBtn);
    connect(viewB1CBtn,SIGNAL(clicked()),this,SLOT(view_signal_clicked()),Qt::QueuedConnection);

    QPushButton *viewB3CBtn = new QPushButton("查看");
    viewB3CBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    viewB3CBtn->setFont(smallFont);
    viewB3CBtn->setDefault(true);
    viewB3CBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(3,10),viewB3CBtn);
    connect(viewB3CBtn,SIGNAL(clicked()),this,SLOT(view_signal_clicked()),Qt::QueuedConnection);

    QPushButton *viewB1B3CBtn = new QPushButton("查看");
    viewB1B3CBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    viewB1B3CBtn->setFont(smallFont);
    viewB1B3CBtn->setDefault(true);
    viewB1B3CBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(4,10),viewB1B3CBtn);
    connect(viewB1B3CBtn,SIGNAL(clicked()),this,SLOT(view_signal_clicked()),Qt::QueuedConnection);

    QPushButton *viewGPSBtn = new QPushButton("查看");
    viewGPSBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    viewGPSBtn->setFont(smallFont);
    viewGPSBtn->setDefault(true);
    viewGPSBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(5,10),viewGPSBtn);
    connect(viewGPSBtn,SIGNAL(clicked()),this,SLOT(view_signal_clicked()),Qt::QueuedConnection);

    QPushButton *viewB1PBtn = new QPushButton("查看");
    viewB1PBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    viewB1PBtn->setFont(smallFont);
    viewB1PBtn->setDefault(true);
    viewB1PBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(6,10),viewB1PBtn);
    connect(viewB1PBtn,SIGNAL(clicked()),this,SLOT(view_signal_clicked()),Qt::QueuedConnection);

    QPushButton *viewB3PBtn = new QPushButton("查看");
    viewB3PBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    viewB3PBtn->setFont(smallFont);
    viewB3PBtn->setDefault(true);
    viewB3PBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(7,10),viewB3PBtn);
    connect(viewB3PBtn,SIGNAL(clicked()),this,SLOT(view_signal_clicked()),Qt::QueuedConnection);

    QPushButton *viewCompatibleBtn = new QPushButton("查看");
    viewCompatibleBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    viewCompatibleBtn->setFont(smallFont);
    viewCompatibleBtn->setDefault(true);
    viewCompatibleBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRn->setIndexWidget(rnModel->index(8,10),viewCompatibleBtn);
    connect(viewCompatibleBtn,SIGNAL(clicked()),this,SLOT(view_signal_clicked()),Qt::QueuedConnection);

    rdModel->item(0,0)->setText("定位模式");
    rdModel->item(0,0)->setFont(font);
    ui->tableViewRd->setSpan(0,0,2,2);

    QPushButton *rdPosBtn = new QPushButton("RDSS单次定位");
    rdPosBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    rdPosBtn->setFont(font);
    rdPosBtn->setDefault(true);
    rdPosBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRd->setIndexWidget(rdModel->index(2,0),rdPosBtn);

    QPushButton *rdContinuePosBtn = new QPushButton("RDSS连续定位");
    rdContinuePosBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    rdContinuePosBtn->setFont(font);
    rdContinuePosBtn->setDefault(true);
    rdContinuePosBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRd->setIndexWidget(rdModel->index(3,0),rdContinuePosBtn);

    QPushButton *rdEmergencyPosBtn = new QPushButton("RDSS紧急定位");
    rdEmergencyPosBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    rdEmergencyPosBtn->setFont(font);
    rdEmergencyPosBtn->setDefault(true);
    rdEmergencyPosBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRd->setIndexWidget(rdModel->index(4,0),rdEmergencyPosBtn);

    QPushButton *rdStopContinuePosBtn = new QPushButton("结束RDSS连续定位");
    rdStopContinuePosBtn->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
    rdStopContinuePosBtn->setFont(font);
    rdStopContinuePosBtn->setDefault(true);
    rdStopContinuePosBtn->setCursor(Qt::PointingHandCursor);
    ui->tableViewRd->setIndexWidget(rdModel->index(5,0),rdStopContinuePosBtn);

    rdModel->item(0,2)->setText("定位结果");
    rdModel->item(0,2)->setFont(font);
    ui->tableViewRd->setSpan(0,2,1,4);

    rdModel->item(1,2)->setText("时间");
    rdModel->item(1,2)->setFont(font);

    rdModel->item(1,3)->setText("经度");
    rdModel->item(1,3)->setFont(font);

    rdModel->item(1,4)->setText("纬度");
    rdModel->item(1,4)->setFont(font);

    rdModel->item(1,5)->setText("高程");
    rdModel->item(1,5)->setFont(font);
}

PosTestWin::~PosTestWin()
{
    delete ui;
}

void PosTestWin::recvBidHandle(const QString &cmd)
{
    QStringList paras = extractParaList(cmd);
    if(paras.size()==2){
        QString mode = paras[0];
        QString code = paras[1];
        if(mode=="1"){
            globalParams::curRnPosMode = "B3"+code;
        }else if(mode=="2"){
            globalParams::curRnPosMode = QString("B1%1+B3%2").arg(code,code);
        }else if(mode=="3"){
            globalParams::curRnPosMode = "B1"+code;
        }else if(mode=="5"){
            globalParams::curRnPosMode = "L1"+code;
        }else if(mode=="8"){
            globalParams::curRnPosMode = QString("B3%1+L1%2").arg(code,code);;
        }else{
            globalParams::curRnPosMode = "";
        }
    }
}

void PosTestWin::recvGgaHandle(const QString &cmd)
{
    QStringList paras = extractParaList(cmd);
    if(paras.size()<14){
        return;
    }
    QString time = paras[0];
    bool ok;
    QString lat = paras[1];
    double dlat = lat.toDouble(&ok);
    QString latitude,longitude,altitude;
    if(ok){
        dlat = degreeSecond2degree(dlat);
        latitude = QString("%1°").arg(dlat,0,'f',8,QChar('0'));
    }else{
        latitude = "";
    }

    QString lon = paras[3];
    double dlon = lon.toDouble(&ok);
    if(ok){
        dlon = degreeSecond2degree(dlon);
        longitude = QString("%1°").arg(dlon,0,'f',8,QChar('0'));
    }else{
        longitude = "";
    }
    if(longitude.size()>latitude.size()){
        while (longitude.size()>latitude.size()) {
            latitude.append(" ");
        }
    }else if(longitude.size()<latitude.size()){
        while (longitude.size()<latitude.size()) {
            longitude.append(" ");
        }
    }
    if(!latitude.isEmpty()){
        latitude.append(paras[2]);
    }
    if(!longitude.isEmpty()){
        longitude.append(paras[4]);
    }
    QString high = paras[8];
    float fhigh = high.toFloat(&ok);
    if(ok){
        fhigh = fhigh + paras[10].toFloat();
        altitude = QString("%1 m").arg(fhigh,0,'f',4,QChar('0'));
    }else{
        altitude = "";
    }

    QString status = paras[5];

    QString satelliteCounts = paras[6];
    QString hdop = paras[7];
    QString vdop;
    if(paras.size()>=15){
        vdop = paras[14];
    }

    int row = -1;
    if(globalParams::curRnPosMode=="B1C"){
        row = 2;
    }else if(globalParams::curRnPosMode=="B3C"){
        row = 3;
    }else if(globalParams::curRnPosMode=="B1C+B3C"){
        row = 4;
    }else if(globalParams::curRnPosMode=="L1C"){
        row = 5;
    }else if(globalParams::curRnPosMode=="B1P"){
        row = 6;
    }else if(globalParams::curRnPosMode=="B3P"){
        row = 7;
    }else if(globalParams::curRnPosMode=="B3C+L1C"){
        row = 8;
    }

    if(status.toInt()!=0){
        if(row >=0){
            rnModel->item(row,2)->setText(time);
            rnModel->item(row,3)->setText(longitude);
            rnModel->item(row,4)->setText(latitude);
            rnModel->item(row,5)->setText(altitude);
            rnModel->item(row,7)->setText(vdop);
            rnModel->item(row,8)->setText(hdop);
            rnModel->item(row,9)->setText(satelliteCounts);
        }
    }else{
        if(row >=0){
            rnModel->item(row,2)->setText("");
            rnModel->item(row,3)->setText("");
            rnModel->item(row,4)->setText("");
            rnModel->item(row,5)->setText("");
            rnModel->item(row,7)->setText("");
            rnModel->item(row,8)->setText("");
            rnModel->item(row,9)->setText("");
        }
    }
}

void PosTestWin::on_btnRdTest_clicked()
{
    ui->gpbRdTest->setVisible(true);
    ui->gpbRnTest->setVisible(false);
    ui->gpbRnSignal->setVisible(false);
}


void PosTestWin::on_btnRnTest_clicked()
{
    ui->gpbRdTest->setVisible(false);
    ui->gpbRnTest->setVisible(true);
    ui->gpbRnSignal->setVisible(false);
}

void PosTestWin::view_signal_clicked()
{
    ui->gpbRdTest->setVisible(false);
    ui->gpbRnTest->setVisible(false);
    ui->gpbRnSignal->setVisible(true);
}

void PosTestWin::on_btnBack_clicked()
{
    ui->gpbRdTest->setVisible(false);
    ui->gpbRnTest->setVisible(true);
    ui->gpbRnSignal->setVisible(false);
}

void PosTestWin::b1cBtn_clicked()
{
    for(int i=2;i<=9;i++){
        rnModel->item(2,i)->setText("");
    }
    ui->tableViewRn->selectRow(2);
    enableOnlyOneViewBtn(2);

    QString cmd = "$CCMSS,Z,1,B1,C,,,,*16\r\n";
    emit writeDataSignal(cmd.toLatin1());
}

void PosTestWin::b3cBtn_clicked()
{
    for(int i=2;i<=9;i++){
        rnModel->item(3,i)->setText("");
    }
    ui->tableViewRn->selectRow(3);
    enableOnlyOneViewBtn(3);

    QString cmd = "$CCMSS,Z,1,B3,C,,,,*14\r\n";
    emit writeDataSignal(cmd.toLatin1());
}

void PosTestWin::b1b3cBtn_clicked()
{
    for(int i=2;i<=9;i++){
        rnModel->item(4,i)->setText("");
    }
    ui->tableViewRn->selectRow(4);
    enableOnlyOneViewBtn(4);

    QString cmd = "$CCMSS,Z,1,B3,C,B1,C,,*24\r\n";
    emit writeDataSignal(cmd.toLatin1());
}

void PosTestWin::gpsBtn_clicked()
{
    for(int i=2;i<=9;i++){
        rnModel->item(5,i)->setText("");
    }
    ui->tableViewRn->selectRow(5);
    enableOnlyOneViewBtn(5);

    QString cmd = "$CCMSS,Z,1,L1,C,,,,*18\r\n";
    emit writeDataSignal(cmd.toLatin1());
}

void PosTestWin::b1pBtn_clicked()
{
    for(int i=2;i<=9;i++){
        rnModel->item(6,i)->setText("");
    }
    ui->tableViewRn->selectRow(6);
    enableOnlyOneViewBtn(6);

    QString cmd = "$CCMSS,Z,1,B1,P,,,,*05\r\n";
    emit writeDataSignal(cmd.toLatin1());
}

void PosTestWin::b3pBtn_clicked()
{
    for(int i=2;i<=9;i++){
        rnModel->item(7,i)->setText("");
    }
    ui->tableViewRn->selectRow(7);
    enableOnlyOneViewBtn(7);

    QString cmd = "$CCMSS,Z,1,B3,P,,,,*07\r\n";
    emit writeDataSignal(cmd.toLatin1());
}

void PosTestWin::compatibleBtn_clicked()
{
    for(int i=2;i<=9;i++){
        rnModel->item(8,i)->setText("");
    }
    ui->tableViewRn->selectRow(8);
    enableOnlyOneViewBtn(8);

    QString cmd = "$CCMSS,Z,1,B3,C,L1,C,,*2A\r\n";
    emit writeDataSignal(cmd.toLatin1());
}

void PosTestWin::enableOnlyOneViewBtn(int row)
{
    QPushButton *viewB1CBtn = qobject_cast<QPushButton *>(ui->tableViewRn->indexWidget(rnModel->index(2,10)));
    QPushButton *viewB3CBtn = qobject_cast<QPushButton *>(ui->tableViewRn->indexWidget(rnModel->index(3,10)));
    QPushButton *viewB1B3CBtn = qobject_cast<QPushButton *>(ui->tableViewRn->indexWidget(rnModel->index(4,10)));
    QPushButton *viewGPSBtn = qobject_cast<QPushButton *>(ui->tableViewRn->indexWidget(rnModel->index(5,10)));
    QPushButton *viewB1PBtn = qobject_cast<QPushButton *>(ui->tableViewRn->indexWidget(rnModel->index(6,10)));
    QPushButton *viewB3PBtn = qobject_cast<QPushButton *>(ui->tableViewRn->indexWidget(rnModel->index(7,10)));
    QPushButton *viewCompatibleBtn = qobject_cast<QPushButton *>(ui->tableViewRn->indexWidget(rnModel->index(8,10)));
    viewB1CBtn->setEnabled(false);
    viewB3CBtn->setEnabled(false);
    viewB1B3CBtn->setEnabled(false);
    viewGPSBtn->setEnabled(false);
    viewB1PBtn->setEnabled(false);
    viewB3PBtn->setEnabled(false);
    viewCompatibleBtn->setEnabled(false);
    switch (row) {
    case 2:
        viewB1CBtn->setEnabled(true);
        break;
    case 3:
        viewB3CBtn->setEnabled(true);
        break;
    case 4:
        viewB1B3CBtn->setEnabled(true);
        break;
    case 5:
        viewGPSBtn->setEnabled(true);
        break;
    case 6:
        viewB1PBtn->setEnabled(true);
        break;
    case 7:
        viewB3PBtn->setEnabled(true);
        break;
    case 8:
        viewCompatibleBtn->setEnabled(true);
        break;
    default:
        break;
    }
}


