#include "rnSignalPainter.h"
#include "ui_rnSignalPainter.h"
#include "globalParams.h"
#include "utils.h"
#include <QPainter>
#include <QDebug>

RnSignalPainter::RnSignalPainter(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::RnSignalPainter)
{
    ui->setupUi(this);
    timerSnrTimeout = new QTimer(this);
    timerSnrTimeout->stop();
    timerSnrTimeout->setInterval(2000);
    connect(timerSnrTimeout,SIGNAL(timeout()),this,SLOT(timerSnrTimeoutHandle()),Qt::QueuedConnection);
    timerSnrTimeout->start();
}

RnSignalPainter::~RnSignalPainter()
{
    delete ui;
}

void RnSignalPainter::recvGsvHandle(const QString &cmd)
{
    QString sys;
    sys = cmd.mid(1,2);
    if(sys=="GB"){
        sys = "BD";
    }
    QStringList paras = extractParaList(cmd);
    if(paras.size()<3){
        return;
    }

    QString feq = "";
    if(paras.size()%4==0){
        feq = paras[paras.size()-1];
    }

    for(int i=0;i<(paras.size()-3)/4;i++){
        QString satNum = paras[3+i*4].trimmed();
        QString cnr = paras[3+i*4+3].trimmed();

        if(satNum.isEmpty() || cnr.isEmpty()){
            continue;
        }

        QPair<QString,QString> key(satNum,feq);
        if(sys=="GP"){
            gpsSnrMap[key] = cnr;
            gpsTimeoutCountMap[key] = 0;
        }else if(sys=="BD"){
            bdSnrMap[key] = cnr;
            bdsTimeoutCountMap[key] = 0;
        }
    }
    update();
}

void RnSignalPainter::timerSnrTimeoutHandle()
{
    for(QPair<QString,QString> key:gpsTimeoutCountMap.keys()){
        gpsTimeoutCountMap[key]++;
        if(gpsTimeoutCountMap[key]>=2){
            gpsSnrMap.remove(key);
            gpsTimeoutCountMap.remove(key);
        }
    }
    for(QPair<QString,QString> key:bdsTimeoutCountMap.keys()){
        bdsTimeoutCountMap[key]++;
        if(bdsTimeoutCountMap[key]>=2){
            bdSnrMap.remove(key);
            bdsTimeoutCountMap.remove(key);
        }
    }
    update();
}

void RnSignalPainter::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setRenderHint(QPainter::TextAntialiasing);
    QPen pen;
    pen.setWidth(1);
    pen.setColor(Qt::black);
    pen.setStyle(Qt::SolidLine);
    painter.setPen(pen);
    QBrush brush;
    brush.setStyle(Qt::SolidPattern);

    int H = this->height(); //绘图区高度
    int W = this->width();

    QFontMetrics fontMetrics = painter.fontMetrics();
    int fontHeight = fontMetrics.height();
    int margin = 5;
    int spacing = 5;

    QString posMode;
    if(globalParams::curRnPosMode=="B3C"){
        posMode = "B3频点C码定位";
    }else if(globalParams::curRnPosMode=="B3P"){
        posMode = "B3频点P码定位";
    }else if(globalParams::curRnPosMode=="B1C+B3C"){
        posMode = "B1+B3频点C码定位";
    }else if(globalParams::curRnPosMode=="B1P+B3P"){
        posMode = "B1+B3频点P码定位";
    }else if(globalParams::curRnPosMode=="B1C"){
        posMode = "B1频点C码定位";
    }else if(globalParams::curRnPosMode=="B1P"){
        posMode = "B1频点P码定位";
    }else if(globalParams::curRnPosMode=="L1C"){
        posMode = "GPS频点C码定位";
    }else if(globalParams::curRnPosMode=="B3C+L1C"){
        posMode = "兼容定位";
    }else{
        posMode = globalParams::curRnPosMode;
    }
    painter.drawText(margin,margin+fontMetrics.ascent(),QString("当前定位模式：%1").arg(posMode));


    //BD坐标轴
    int yb0 = (H-margin*2)/2-margin-fontMetrics.height();
    QString tmp = "北斗卫星信号";
    int xb0 = margin*2 + fontMetrics.horizontalAdvance(tmp);
    int hb = yb0 - margin*2 - fontMetrics.height()*2;
    int wb = W - margin*2 - fontMetrics.horizontalAdvance(tmp);
    painter.drawText(margin,yb0-hb/2,tmp);
    painter.drawLine(xb0,yb0,xb0+wb,yb0);
    painter.drawLine(xb0,yb0,xb0,yb0-hb);
    //GP坐标轴
    int yg0 = H-margin-fontMetrics.height();
    tmp = "GPS卫星信号";
    int xg0 = xb0;
    int hg = hb;
    int wg = wb;
    painter.drawText(margin,yg0-hg/2,tmp);
    painter.drawLine(xg0,yg0,xg0+wg,yg0);
    painter.drawLine(xg0,yg0,xg0,yg0-hg);

    int sw = fontMetrics.horizontalAdvance("00.0");

    //BD信号
    int xb = xb0+margin;
    for(const QPair<QString,QString> &key:bdSnrMap.keys()){
        double strength = bdSnrMap.value(key).toDouble();
        QString satNum = key.first;
        QString feq = key.second;
        int red = 255 - feq.toInt(nullptr,16)*10;
        QColor color = QColor(red,0,0);
        brush.setColor(color);
        painter.setBrush(brush);

        xb = xb + margin;

        int w0 = fontMetrics.horizontalAdvance(satNum);//宽度
        painter.drawText(xb+(sw-w0)/2,yb0+fontMetrics.ascent(),satNum);

        //int h = std::min(strength,50.0);
        int h = std::min(hb/50.0*strength,hb*1.0);
        int y = yb0-h;
        QRect rect(xb,y,sw,h);
        painter.drawRect(rect);

        QString str = QString("%1").arg(strength,0,'g',3);
        int w1 = fontMetrics.horizontalAdvance(str);//宽度
        painter.drawText(xb+(sw-w1)/2,y-margin/2-fontMetrics.descent(),str);
        xb = xb + sw;
    }

    //GPS信号
    int xg = xb0+margin;
    for(const QPair<QString,QString> &key:gpsSnrMap.keys()){
        double strength = gpsSnrMap.value(key).toDouble();
        QString satNum = key.first;
        QString feq = key.second;
        int green = 255 - feq.toInt(nullptr,16)*10;
        QColor color = QColor(0,green,0);
        brush.setColor(color);
        painter.setBrush(brush);

        xg = xg + margin;

        int w0 = fontMetrics.horizontalAdvance(satNum);//宽度
        painter.drawText(xg+(sw-w0)/2,yg0+fontMetrics.ascent(),satNum);

        //int h = std::min(strength,50.0);
        int h = std::min(hb/50.0*strength,hg*1.0);
        int y = yg0-h;
        QRect rect(xg,y,sw,h);
        painter.drawRect(rect);

        QString str = QString("%1").arg(strength,0,'g',3);
        int w1 = fontMetrics.horizontalAdvance(str);//宽度
        painter.drawText(xg+(sw-w1)/2,y-margin/2-fontMetrics.descent(),str);
        xg = xg + sw;
    }

    return QWidget::paintEvent(event);
}
